@@ -56,12 +56,6 @@ typedef struct
 	/* Non-scanned attributes */
 	gboolean			fake;	/* Whether or not the AP is from a scan */
 	gboolean			broadcast;	/* Whether or not the AP is broadcasting (hidden) */
-	gboolean			user_created;	/* Whether or not the AP was created
-										 * by the user with "Create network..."
-										 * A subset of Ad-Hoc mode.  user_created
-										 * implies Ad-Hoc, but not necessarily
-										 * the other way around.
-										 */
 	glong				last_seen;	/* Last time the AP was seen in a scan in seconds */
 
 	/* Things from user prefs/NetworkManagerInfo */
@@ -592,9 +586,9 @@ nm_ap_new_fake_from_connection (NMConnection *connection)
 
 	mode = nm_setting_wireless_get_mode (s_wireless);
 	if (mode) {
-		if (!strcmp (mode, "infrastructure"))
+		if (!strcmp (mode, NM_SETTING_WIRELESS_MODE_INFRA))
 			nm_ap_set_mode (ap, NM_802_11_MODE_INFRA);
-		else if (!strcmp (mode, "adhoc"))
+		else if (!strcmp (mode, NM_SETTING_WIRELESS_MODE_ADHOC))
 			nm_ap_set_mode (ap, NM_802_11_MODE_ADHOC);
 		else
 			goto error;
@@ -1098,28 +1092,6 @@ void nm_ap_set_last_seen (NMAccessPoint *ap, const glong last_seen)
 }
 
 
-/*
- * Get/Set functions to indicate that an access point is
- * user-created, ie whether or not its a network filled with
- * information from the user and intended to create a new Ad-Hoc
- * wireless network.
- *
- */
-gboolean nm_ap_get_user_created (const NMAccessPoint *ap)
-{
-	g_return_val_if_fail (NM_IS_AP (ap), FALSE);
-
-	return NM_AP_GET_PRIVATE (ap)->user_created;
-}
-
-void nm_ap_set_user_created (NMAccessPoint *ap, gboolean user_created)
-{
-	g_return_if_fail (NM_IS_AP (ap));
-
-	NM_AP_GET_PRIVATE (ap)->user_created = user_created;
-}
-
-
 /*
  * Get/Set functions for user address list
  *
@@ -1239,9 +1211,9 @@ nm_ap_check_compatible (NMAccessPoint *self,
 
 	mode = nm_setting_wireless_get_mode (s_wireless);
 	if (mode) {
-		if (!strcmp (mode, "infrastructure") && (priv->mode != NM_802_11_MODE_INFRA))
+		if (!strcmp (mode, NM_SETTING_WIRELESS_MODE_INFRA) && (priv->mode != NM_802_11_MODE_INFRA))
 			return FALSE;
-		if (!strcmp (mode, "adhoc") && (priv->mode != NM_802_11_MODE_ADHOC))
+		if (!strcmp (mode, NM_SETTING_WIRELESS_MODE_ADHOC) && (priv->mode != NM_802_11_MODE_ADHOC))
 			return FALSE;
 	}
 
